@@ -1,7 +1,8 @@
-# Copyright (c) Open-MMLab. All rights reserved.
+# Copyright (c) OpenMMLab. All rights reserved.
 import copy
 import math
 import warnings
+from typing import Dict, List, Optional, Union
 
 import numpy as np
 import torch
@@ -13,14 +14,49 @@ from mmcv.utils import Registry, build_from_cfg, get_logger, print_log
 INITIALIZERS = Registry('initializer')
 
 
-def constant_init(module, val, bias=0):
+def update_init_info(module: nn.Module, init_info: str) -> None:
+    """Update the `_params_init_info` in the module if the value of parameters
+    are changed.
+
+    Args:
+        module (obj:`nn.Module`): The module of PyTorch with a user-defined
+            attribute `_params_init_info` which records the initialization
+            information.
+        init_info (str): The string that describes the initialization.
+    """
+    assert hasattr(
+        module,
+        '_params_init_info'), f'Can not find `_params_init_info` in {module}'
+    for name, param in module.named_parameters():
+
+        assert param in module._params_init_info, (
+            f'Find a new :obj:`Parameter` '
+            f'named `{name}` during executing the '
+            f'`init_weights` of '
+            f'`{module.__class__.__name__}`. '
+            f'Please do not add or '
+            f'replace parameters during executing '
+            f'the `init_weights`. ')
+
+        # The parameter has been changed during executing the
+        # `init_weights` of module
+        mean_value = param.data.mean()
+        if module._params_init_info[param]['tmp_mean_value'] != mean_value:
+            module._params_init_info[param]['init_info'] = init_info
+            module._params_init_info[param]['tmp_mean_value'] = mean_value
+
+
+def constant_init(module: nn.Module, val: float, bias: float = 0) -> None:
     if hasattr(module, 'weight') and module.weight is not None:
         nn.init.constant_(module.weight, val)
     if hasattr(module, 'bias') and module.bias is not None:
         nn.init.constant_(module.bias, bias)
 
 
-def xavier_init(module, gain=1, bias=0, distribution='normal'):
+def xavier_init(module: nn.Module,
+                gain: float = 1,
+                bias: float = 0,
+                distribution: str = 'normal') -> None:
     assert distribution in ['uniform', 'normal']
     if hasattr(module, 'weight') and module.weight is not None:
         if distribution == 'uniform':
@@ -31,7 +67,10 @@ def xavier_init(module, gain=1, bias=0, distribution='normal'):
         nn.init.constant_(module.bias, bias)
 
 
-def normal_init(module, mean=0, std=1, bias=0):
+def normal_init(module: nn.Module,
+                mean: float = 0,
+                std: float = 1,
+                bias: float = 0) -> None:
     if hasattr(module, 'weight') and module.weight is not None:
         nn.init.normal_(module.weight, mean, std)
     if hasattr(module, 'bias') and module.bias is not None:
@@ -50,19 +89,22 @@ def trunc_normal_init(module: nn.Module,
         nn.init.constant_(module.bias, bias)  # type: ignore
 
 
-def uniform_init(module, a=0, b=1, bias=0):
+def uniform_init(module: nn.Module,
+                 a: float = 0,
+                 b: float = 1,
+                 bias: float = 0) -> None:
     if hasattr(module, 'weight') and module.weight is not None:
         nn.init.uniform_(module.weight, a, b)
     if hasattr(module, 'bias') and module.bias is not None:
         nn.init.constant_(module.bias, bias)
 
 
-def kaiming_init(module,
-                 a=0,
-                 mode='fan_out',
-                 nonlinearity='relu',
-                 bias=0,
-                 distribution='normal'):
+def kaiming_init(module: nn.Module,
+                 a: float = 0,
+                 mode: str = 'fan_out',
+                 nonlinearity: str = 'relu',
+                 bias: float = 0,
+                 distribution: str = 'normal') -> None:
     assert distribution in ['uniform', 'normal']
     if hasattr(module, 'weight') and module.weight is not None:
         if distribution == 'uniform':
@@ -75,7 +117,7 @@ def kaiming_init(module,
         nn.init.constant_(module.bias, bias)
 
 
-def caffe2_xavier_init(module, bias=0):
+def caffe2_xavier_init(module: nn.Module, bias: float = 0) -> None:
     # `XavierFill` in Caffe2 corresponds to `kaiming_uniform_` in PyTorch
     # Acknowledgment to FAIR's internal code
     kaiming_init(
@@ -87,15 +129,23 @@ def caffe2_xavier_init(module, bias=0):
         distribution='uniform')
 
 
-def bias_init_with_prob(prior_prob):
+def bias_init_with_prob(prior_prob: float) -> float:
     """initialize conv/fc bias value according to a given probability value."""
     bias_init = float(-np.log((1 - prior_prob) / prior_prob))
     return bias_init
 
 
-class BaseInit(object):
+def _get_bases_name(m: nn.Module) -> List[str]:
+    return [b.__name__ for b in m.__class__.__bases__]
 
-    def __init__(self, *, bias=0, bias_prob=None, layer=None):
+
+class BaseInit:
+
+    def __init__(self,
+                 *,
+                 bias: float = 0,
+                 bias_prob: Optional[float] = None,
+                 layer: Union[str, List, None] = None):
         self.wholemodule = False
         if not isinstance(bias, (int, float)):
             raise TypeError(f'bias must be a number, but got a {type(bias)}')
@@ -111,15 +161,17 @@ class BaseInit(object):
                     but got a {type(layer)}')
         else:
             layer = []
-            warnings.warn(
-                'init_cfg without layer key, if you do not define override'
-                ' key either, this init_cfg will do nothing')
+
         if bias_prob is not None:
             self.bias = bias_init_with_prob(bias_prob)
         else:
             self.bias = bias
         self.layer = [layer] if isinstance(layer, str) else layer
 
+    def _get_init_info(self) -> str:
+        info = f'{self.__class__.__name__}, bias={self.bias}'
+        return info
+
 
 @INITIALIZERS.register_module(name='Constant')
 class ConstantInit(BaseInit):
@@ -127,29 +179,35 @@ class ConstantInit(BaseInit):
 
     Args:
         val (int | float): the value to fill the weights in the module with
-        bias (int | float): the value to fill the bias or
-        define initialization type for bias. Defaults to 0.
+        bias (int | float): the value to fill the bias. Defaults to 0.
         bias_prob (float, optional): the probability for bias initialization.
             Defaults to None.
         layer (str | list[str], optional): the layer will be initialized.
             Defaults to None.
     """
 
-    def __init__(self, val, **kwargs):
+    def __init__(self, val: Union[int, float], **kwargs):
         super().__init__(**kwargs)
         self.val = val
 
-    def __call__(self, module):
+    def __call__(self, module: nn.Module) -> None:
 
         def init(m):
             if self.wholemodule:
                 constant_init(m, self.val, self.bias)
             else:
                 layername = m.__class__.__name__
-                if layername in self.layer:
+                basesname = _get_bases_name(m)
+                if len(set(self.layer) & set([layername] + basesname)):
                     constant_init(m, self.val, self.bias)
 
         module.apply(init)
+        if hasattr(module, '_params_init_info'):
+            update_init_info(module, init_info=self._get_init_info())
+
+    def _get_init_info(self) -> str:
+        info = f'{self.__class__.__name__}: val={self.val}, bias={self.bias}'
+        return info
 
 
 @INITIALIZERS.register_module(name='Xavier')
@@ -161,8 +219,7 @@ class XavierInit(BaseInit):
 
     Args:
         gain (int | float): an optional scaling factor. Defaults to 1.
-        bias (int | float): the value to fill the bias or define
-            initialization type for bias. Defaults to 0.
+        bias (int | float): the value to fill the bias. Defaults to 0.
         bias_prob (float, optional): the probability for bias initialization.
             Defaults to None.
         distribution (str): distribution either be ``'normal'``
@@ -171,22 +228,33 @@ class XavierInit(BaseInit):
             Defaults to None.
     """
 
-    def __init__(self, gain=1, distribution='normal', **kwargs):
+    def __init__(self,
+                 gain: float = 1,
+                 distribution: str = 'normal',
+                 **kwargs):
         super().__init__(**kwargs)
         self.gain = gain
         self.distribution = distribution
 
-    def __call__(self, module):
+    def __call__(self, module: nn.Module) -> None:
 
         def init(m):
             if self.wholemodule:
                 xavier_init(m, self.gain, self.bias, self.distribution)
             else:
                 layername = m.__class__.__name__
-                if layername in self.layer:
+                basesname = _get_bases_name(m)
+                if len(set(self.layer) & set([layername] + basesname)):
                     xavier_init(m, self.gain, self.bias, self.distribution)
 
         module.apply(init)
+        if hasattr(module, '_params_init_info'):
+            update_init_info(module, init_info=self._get_init_info())
+
+    def _get_init_info(self) -> str:
+        info = f'{self.__class__.__name__}: gain={self.gain}, ' \
+               f'distribution={self.distribution}, bias={self.bias}'
+        return info
 
 
 @INITIALIZERS.register_module(name='Normal')
@@ -198,8 +266,7 @@ class NormalInit(BaseInit):
         mean (int | float):the mean of the normal distribution. Defaults to 0.
         std (int | float): the standard deviation of the normal distribution.
             Defaults to 1.
-        bias (int | float): the value to fill the bias or define
-            initialization type for bias. Defaults to 0.
+        bias (int | float): the value to fill the bias. Defaults to 0.
         bias_prob (float, optional): the probability for bias initialization.
             Defaults to None.
         layer (str | list[str], optional): the layer will be initialized.
@@ -207,23 +274,30 @@ class NormalInit(BaseInit):
 
     """
 
-    def __init__(self, mean=0, std=1, **kwargs):
+    def __init__(self, mean: float = 0, std: float = 1, **kwargs):
         super().__init__(**kwargs)
         self.mean = mean
         self.std = std
 
-    def __call__(self, module):
+    def __call__(self, module: nn.Module) -> None:
 
         def init(m):
             if self.wholemodule:
                 normal_init(m, self.mean, self.std, self.bias)
             else:
                 layername = m.__class__.__name__
-                for layer_ in self.layer:
-                    if layername == layer_:
-                        normal_init(m, self.mean, self.std, self.bias)
+                basesname = _get_bases_name(m)
+                if len(set(self.layer) & set([layername] + basesname)):
+                    normal_init(m, self.mean, self.std, self.bias)
 
         module.apply(init)
+        if hasattr(module, '_params_init_info'):
+            update_init_info(module, init_info=self._get_init_info())
+
+    def _get_init_info(self) -> str:
+        info = f'{self.__class__.__name__}: mean={self.mean},' \
+               f' std={self.std}, bias={self.bias}'
+        return info
 
 
 @INITIALIZERS.register_module(name='TruncNormal')
@@ -238,8 +312,7 @@ class TruncNormalInit(BaseInit):
             Defaults to 1.
         a (float): The minimum cutoff value.
         b ( float): The maximum cutoff value.
-        bias (float): the value to fill the bias or define
-            initialization type for bias. Defaults to 0.
+        bias (float): the value to fill the bias. Defaults to 0.
         bias_prob (float, optional): the probability for bias initialization.
             Defaults to None.
         layer (str | list[str], optional): the layer will be initialized.
@@ -267,12 +340,19 @@ class TruncNormalInit(BaseInit):
                                   self.bias)
             else:
                 layername = m.__class__.__name__
-                for layer_ in self.layer:
-                    if layername == layer_:
-                        trunc_normal_init(m, self.mean, self.std, self.a,
-                                          self.b, self.bias)
+                basesname = _get_bases_name(m)
+                if len(set(self.layer) & set([layername] + basesname)):
+                    trunc_normal_init(m, self.mean, self.std, self.a, self.b,
+                                      self.bias)
 
         module.apply(init)
+        if hasattr(module, '_params_init_info'):
+            update_init_info(module, init_info=self._get_init_info())
+
+    def _get_init_info(self):
+        info = f'{self.__class__.__name__}: a={self.a}, b={self.b},' \
+               f' mean={self.mean}, std={self.std}, bias={self.bias}'
+        return info
 
 
 @INITIALIZERS.register_module(name='Uniform')
@@ -285,35 +365,42 @@ class UniformInit(BaseInit):
             Defaults to 0.
         b (int | float): the upper bound of the uniform distribution.
             Defaults to 1.
-        bias (int | float): the value to fill the bias or define
-            initialization type for bias. Defaults to 0.
+        bias (int | float): the value to fill the bias. Defaults to 0.
         bias_prob (float, optional): the probability for bias initialization.
             Defaults to None.
         layer (str | list[str], optional): the layer will be initialized.
             Defaults to None.
     """
 
-    def __init__(self, a=0, b=1, **kwargs):
+    def __init__(self, a: float = 0., b: float = 1., **kwargs):
         super().__init__(**kwargs)
         self.a = a
         self.b = b
 
-    def __call__(self, module):
+    def __call__(self, module: nn.Module) -> None:
 
         def init(m):
             if self.wholemodule:
                 uniform_init(m, self.a, self.b, self.bias)
             else:
                 layername = m.__class__.__name__
-                if layername in self.layer:
+                basesname = _get_bases_name(m)
+                if len(set(self.layer) & set([layername] + basesname)):
                     uniform_init(m, self.a, self.b, self.bias)
 
         module.apply(init)
+        if hasattr(module, '_params_init_info'):
+            update_init_info(module, init_info=self._get_init_info())
+
+    def _get_init_info(self) -> str:
+        info = f'{self.__class__.__name__}: a={self.a},' \
+               f' b={self.b}, bias={self.bias}'
+        return info
 
 
 @INITIALIZERS.register_module(name='Kaiming')
 class KaimingInit(BaseInit):
-    r"""Initialize module paramters with the valuse according to the method
+    r"""Initialize module parameters with the values according to the method
     described in `Delving deep into rectifiers: Surpassing human-level
     performance on ImageNet classification - He, K. et al. (2015).
     <https://www.cv-foundation.org/openaccess/content_iccv_2015/
@@ -329,8 +416,7 @@ class KaimingInit(BaseInit):
         nonlinearity (str): the non-linear function (`nn.functional` name),
             recommended to use only with ``'relu'`` or ``'leaky_relu'`` .
             Defaults to 'relu'.
-        bias (int | float): the value to fill the bias or define
-            initialization type for bias. Defaults to 0.
+        bias (int | float): the value to fill the bias. Defaults to 0.
         bias_prob (float, optional): the probability for bias initialization.
             Defaults to None.
         distribution (str): distribution either be ``'normal'`` or
@@ -340,10 +426,10 @@ class KaimingInit(BaseInit):
     """
 
     def __init__(self,
-                 a=0,
-                 mode='fan_out',
-                 nonlinearity='relu',
-                 distribution='normal',
+                 a: float = 0,
+                 mode: str = 'fan_out',
+                 nonlinearity: str = 'relu',
+                 distribution: str = 'normal',
                  **kwargs):
         super().__init__(**kwargs)
         self.a = a
@@ -351,7 +437,7 @@ class KaimingInit(BaseInit):
         self.nonlinearity = nonlinearity
         self.distribution = distribution
 
-    def __call__(self, module):
+    def __call__(self, module: nn.Module) -> None:
 
         def init(m):
             if self.wholemodule:
@@ -359,11 +445,20 @@ class KaimingInit(BaseInit):
                              self.bias, self.distribution)
             else:
                 layername = m.__class__.__name__
-                if layername in self.layer:
+                basesname = _get_bases_name(m)
+                if len(set(self.layer) & set([layername] + basesname)):
                     kaiming_init(m, self.a, self.mode, self.nonlinearity,
                                  self.bias, self.distribution)
 
         module.apply(init)
+        if hasattr(module, '_params_init_info'):
+            update_init_info(module, init_info=self._get_init_info())
+
+    def _get_init_info(self) -> str:
+        info = f'{self.__class__.__name__}: a={self.a}, mode={self.mode}, ' \
+               f'nonlinearity={self.nonlinearity}, ' \
+               f'distribution ={self.distribution}, bias={self.bias}'
+        return info
 
 
 @INITIALIZERS.register_module(name='Caffe2Xavier')
@@ -378,12 +473,12 @@ class Caffe2XavierInit(KaimingInit):
             distribution='uniform',
             **kwargs)
 
-    def __call__(self, module):
+    def __call__(self, module: nn.Module) -> None:
         super().__call__(module)
 
 
 @INITIALIZERS.register_module(name='Pretrained')
-class PretrainedInit(object):
+class PretrainedInit:
     """Initialize module by loading a pretrained model.
 
     Args:
@@ -397,12 +492,15 @@ class PretrainedInit(object):
         map_location (str): map tensors into proper locations.
     """
 
-    def __init__(self, checkpoint, prefix=None, map_location=None):
+    def __init__(self,
+                 checkpoint: str,
+                 prefix: Optional[str] = None,
+                 map_location: Optional[str] = None):
         self.checkpoint = checkpoint
         self.prefix = prefix
         self.map_location = map_location
 
-    def __call__(self, module):
+    def __call__(self, module: nn.Module) -> None:
         from mmcv.runner import (_load_checkpoint_with_prefix, load_checkpoint,
                                  load_state_dict)
         logger = get_logger('mmcv')
@@ -422,8 +520,17 @@ class PretrainedInit(object):
                 self.prefix, self.checkpoint, map_location=self.map_location)
             load_state_dict(module, state_dict, strict=False, logger=logger)
 
+        if hasattr(module, '_params_init_info'):
+            update_init_info(module, init_info=self._get_init_info())
 
-def _initialize(module, cfg, wholemodule=False):
+    def _get_init_info(self) -> str:
+        info = f'{self.__class__.__name__}: load from {self.checkpoint}'
+        return info
+
+
+def _initialize(module: nn.Module,
+                cfg: Dict,
+                wholemodule: bool = False) -> None:
     func = build_from_cfg(cfg, INITIALIZERS)
     # wholemodule flag is for override mode, there is no layer key in override
     # and initializer will give init values for the whole module with the name
@@ -432,7 +539,8 @@ def _initialize(module, cfg, wholemodule=False):
     func(module)
 
 
-def _initialize_override(module, override, cfg):
+def _initialize_override(module: nn.Module, override: Union[Dict, List],
+                         cfg: Dict) -> None:
     if not isinstance(override, (dict, list)):
         raise TypeError(f'override must be a dict or a list of dict, \
                 but got {type(override)}')
@@ -446,7 +554,7 @@ def _initialize_override(module, override, cfg):
         if name is None:
             raise ValueError('`override` must contain the key "name",'
                              f'but got {cp_override}')
-        # if override only has name kay, it means use args in init_cfg
+        # if override only has name key, it means use args in init_cfg
         if not cp_override:
             cp_override.update(cfg)
         # if override has name key and other args except type key, it will
@@ -462,8 +570,8 @@ def _initialize_override(module, override, cfg):
                                f'but init_cfg is {cp_override}.')
 
 
-def initialize(module, init_cfg):
-    """Initialize a module.
+def initialize(module: nn.Module, init_cfg: Union[Dict, List[dict]]) -> None:
+    r"""Initialize a module.
 
     Args:
         module (``torch.nn.Module``): the module will be initialized.
@@ -471,6 +579,7 @@ def initialize(module, init_cfg):
             define initializer. OpenMMLab has implemented 6 initializers
             including ``Constant``, ``Xavier``, ``Normal``, ``Uniform``,
             ``Kaiming``, and ``Pretrained``.
+
     Example:
         >>> module = nn.Linear(2, 3, bias=True)
         >>> init_cfg = dict(type='Constant', layer='Linear', val =1 , bias =2)
